/* OrenNayar diffuse BRDF,
 * N, I, and L should be passed as-is directly from the shader.
 * rough: 0=lambert, larger is rougher.
 */

float OrenNayarBRDF( normal N; vector I, L; float rough )
{
	normal Normal = faceforward( normalize(N), I, N );
	vector View = normalize( -I );
	vector Light = normalize( L );
	vector Half = normalize( View + Light );
	
	float nh = Normal . Half;
	float nv = Normal . View;
	float nl = Normal . Light;
	float vh = View . Half;
	
	
	float Lit_A = acos( nl );
	float View_A = acos( nv );
	
	
	vector Lit_B = 0; 
	Lit_B[0] = Light[0] - (nl * Normal[0]);
	Lit_B[1] = Light[1] - (nl * Normal[1]);
	Lit_B[2] = Light[2] - (nl * Normal[2]);
	Lit_B = normalize( Lit_B );
	
	vector View_B = 0;
	View_B[0] = View[0] - (nv * Normal[0]);
	View_B[1] = View[1] - (nv * Normal[1]);
	View_B[2] = View[2] - (nv * Normal[2]);
	View_B = normalize( View_B );
	
	
	float t = Lit_B . View_B;
	if( t < 0 )
		t = 0;
	
	
	float a, b;
	if( Lit_A > View_A )
	{
		a = Lit_A;
		b = View_A;
	}
	else
	{
		a = View_A;
		b = Lit_A;
	}
	
	
	float A = 1.0 - (0.5 * ((rough * rough) / ((rough * rough) + 0.33)));
	float B = 0.45 * ((rough * rough) / ((rough * rough) + 0.09));
	
	return nl * ( A + ( B * t * sin(a) * tan(b) ) );
}
