/* Blinn specular BRDF.
 * N, I, and L should be passed as-is directly from the shader.
 * rough: closer to 0 is shiny, closer to 1 is rough.
 * fresnel: the index of refection, determining the fresnel effect.
 */

float BlinnBRDF( normal N; vector I, L; float rough, fresnel )
{
	normal Normal = faceforward( normalize(N), I, N );
	vector View = normalize( -I );
	vector Light = normalize( L );
	vector Half = normalize( View + Light );
	
	
	
	// D
	float D = Normal . Half;
	D = ((D*D) * ((rough*rough)-1)) + 1;
	
	if( D == 0 )
		D = 0.0001;
	D = (rough*rough) / D;
	D = D*D;
	
	
	
	// G
	float temp = View . Half;
	if( temp == 0 )
		temp = 0.0001;
		
	float Ga = 2 * (Normal.Half) * (Normal.View) / temp;
	float Gb = 2 * (Normal.Half) * (Normal.Light) / temp;
	
	float G = min( 1.0, Ga, Gb );
	
	
	
	// F
	float c = View . Half;
	float g = sqrt( (fresnel*fresnel) + (c*c) - 1 );
	
	temp = (g+c)*(g+c);
	if( temp == 0 )
		temp = 0.0001;
	
	float Fa = ((g-c)*(g-c)) / temp;
	
	temp = ((c*(g-c))+1);
	temp = temp*temp;
	if( temp == 0 )
		temp = 0.0001;
	
	float Fb = 1 + ((((c*(g+c))+1)*((c*(g+c))+1)) / temp );
	float F = Fa*Fb;
	
	
	
	return D*G*F;
}
